﻿#region Copyright notice and license
// Protocol Buffers - Google's data interchange format
// Copyright 2015 Google Inc.  All rights reserved.
// https://developers.google.com/protocol-buffers/
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//     * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above
// copyright notice, this list of conditions and the following disclaimer
// in the documentation and/or other materials provided with the
// distribution.
//     * Neither the name of Google Inc. nor the names of its
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#endregion

using System;
using System.Reflection;
using Google.Protobuf.Descriptors;

namespace Google.Protobuf.FieldAccess
{
    /// <summary>
    /// Base class for field accessors.
    /// </summary>
    internal abstract class FieldAccessorBase : IFieldAccessor
    {
        private readonly Func<object, object> getValueDelegate;
        private readonly FieldDescriptor descriptor;

        internal FieldAccessorBase(Type type, string propertyName, FieldDescriptor descriptor)
        {
            PropertyInfo property = type.GetProperty(propertyName);
            if (property == null || !property.CanRead)
            {
                throw new ArgumentException("Not all required properties/methods available");
            }
            this.descriptor = descriptor;
            getValueDelegate = ReflectionUtil.CreateFuncObjectObject(property.GetGetMethod());
        }

        public FieldDescriptor Descriptor { get { return descriptor; } }

        public object GetValue(object message)
        {
            return getValueDelegate(message);
        }

        public abstract void Clear(object message);
        public abstract void SetValue(object message, object value);
    }
}
